

namespace Pdfizer
{
	using System;
	using System.IO;
	using System.Collections;

	using NTidy;
	
	using iTextSharp.text;
	using iTextSharp.text.pdf;
	
	/// <summary>
	/// TODO - Add class summary
	/// </summary>
	/// <remarks>
	/// 	created by - dehalleux
	/// 	created on - 13/01/2004 11:47:39
	/// </remarks>
	public class AnyHtmlElement : HtmlElement
	{
		private ITextElementArray parent;
		private Hashtable rules;
		
		public AnyHtmlElement(
			HtmlToPdfConverter converter,
			ITextElementArray parent
			)
		:base(converter)
		{
			this.parent = parent;
			
			this.rules=new Hashtable();
			
			// adding rules
			this.rules["p"]=new ParaHtmlElement(Converter);
			this.rules["pre"]=new ParaHtmlElement(Converter);
			this.rules["div"]=new ParaHtmlElement(Converter);
			
			this.rules["verbatim"]=new VerbatimHtmlElement(Converter);
			
			this.rules["b"]=new PhraseHtmlElement(Converter);
			this.rules["em"]=new PhraseHtmlElement(Converter);
			this.rules["tt"]=new PhraseHtmlElement(Converter);
			this.rules["it"]=new PhraseHtmlElement(Converter);
			this.rules["code"]=new PhraseHtmlElement(Converter);
			this.rules["span"]=new PhraseHtmlElement(Converter);
			
			this.rules["ul"]=new ListHtmlElement(Converter);
			this.rules["ol"]=new ListHtmlElement(Converter);
			this.rules["li"]=new ListItemHtmlElement(Converter);
			
			this.rules["h1"]=new HeaderHtmlElement(Converter);
			this.rules["h2"]=new HeaderHtmlElement(Converter);
			this.rules["h3"]=new HeaderHtmlElement(Converter);
			this.rules["h4"]=new HeaderHtmlElement(Converter);
			this.rules["h5"]=new HeaderHtmlElement(Converter);
			this.rules["h6"]=new HeaderHtmlElement(Converter);

			this.rules["img"]=new ImgHtmlElement(Converter);

			this.Rules["a"]= new AHtmlElement(Converter);
		}
		
		public IDictionary Rules
		{
			get
			{
				return rules;
			}
		}
		
		public ITextElementArray Parent
		{
			get
			{
				return parent;
			}
		}
				
		public TidyNode Convert(TidyNode el)
		{
			if (el==null)
				throw new ArgumentNullException("el");
			
			if (Rules.Contains(el.Name.ToLower()))
			{
				IHtmlElement conv = Rules[el.Name.ToLower()] as IHtmlElement;
				conv.Convert(el);
				
				if (Parent==null)
					Converter.Doc.Add(conv.Chunk);
				else
					Parent.Add(conv.Chunk);
				
				return el.NextSibling;
			}
			else // this tag is not supported.
			{
				// add inner html as text
				IElement chunk = Converter.TextProcessor.Process(el.Value);
				if (Parent==null)
					Converter.Doc.Add(chunk);
				else
					Parent.Add(chunk);
				
				return el.NextSibling;
			}
		}
		
		public void ConvertChilds(TidyNode el)
		{
			if (el==null)
				throw new ArgumentNullException("el");
			TidyNode node = el.FirstChild;
			while (node != null)
			{
				if (node.IsText)
				{
					if (this.Parent!=null)
						this.Parent.Add( new Chunk( node.Value ) );
					else
						this.Converter.Doc.Add( new Chunk(node.Value));
					node = node.NextSibling;
				}
				else
					node = Convert(node);
			}			
		}
	}
}
